@@ -1,8 +1,15 @@
 import io
 
-from palabra_ai import BufferReader
-from palabra_ai import (PalabraAI, Config, SourceLang, TargetLang,
-                        BufferWriter, AR, EN)
+from palabra_ai import (
+    AR,
+    EN,
+    BufferReader,
+    BufferWriter,
+    Config,
+    PalabraAI,
+    SourceLang,
+    TargetLang,
+)
 from palabra_ai.internal.audio import convert_any_to_pcm16
 
 if __name__ == "__main__":
@@ -14,6 +21,8 @@ if __name__ == "__main__":
     writer = BufferWriter(es_buffer)
     cfg = Config(SourceLang(AR, reader), [TargetLang(EN, writer)])
     palabra.run(cfg)
-    print(f"Translated audio written to buffer with size: {es_buffer.getbuffer().nbytes} bytes")
+    print(
+        f"Translated audio written to buffer with size: {es_buffer.getbuffer().nbytes} bytes"
+    )
     with open("./ar2en_out.wav", "wb") as f:
-        f.write(es_buffer.getbuffer())
\ No newline at end of file
+        f.write(es_buffer.getbuffer())
