@@ -44,10 +44,12 @@ from palabra_ai.constant import (
     VAD_THRESHOLD_DEFAULT,
     WEBRTC_MODE_CHANNELS,
     WEBRTC_MODE_CHUNK_DURATION_MS,
-    WEBRTC_MODE_SAMPLE_RATE,
+    WEBRTC_MODE_INPUT_SAMPLE_RATE,
+    WEBRTC_MODE_OUTPUT_SAMPLE_RATE,
     WS_MODE_CHANNELS,
     WS_MODE_CHUNK_DURATION_MS,
-    WS_MODE_SAMPLE_RATE,
+    WS_MODE_INPUT_SAMPLE_RATE,
+    WS_MODE_OUTPUT_SAMPLE_RATE,
 )
 from palabra_ai.exc import ConfigurationError
 from palabra_ai.lang import Language, is_valid_source_language, is_valid_target_language
@@ -102,6 +104,16 @@ def serialize_language(lang: Language) -> str:
     return lang.bcp47
 
 
+def serialize_language_source(lang: Language) -> str:
+    """Serialize language for source (recognition) - uses source_code"""
+    return lang.source_code
+
+
+def serialize_language_target(lang: Language) -> str:
+    """Serialize language for target (translation) - uses target_code"""
+    return lang.target_code
+
+
 LanguageField = Annotated[
     Language, BeforeValidator(validate_language), PlainSerializer(serialize_language)
 ]
@@ -109,25 +121,26 @@ LanguageField = Annotated[
 SourceLanguageField = Annotated[
     Language,
     BeforeValidator(validate_source_language),
-    PlainSerializer(serialize_language),
+    PlainSerializer(serialize_language_source),
 ]
 
 TargetLanguageField = Annotated[
     Language,
     BeforeValidator(validate_target_language),
-    PlainSerializer(serialize_language),
+    PlainSerializer(serialize_language_target),
 ]
 
 
 class IoMode(BaseModel):
     name: str
-    sample_rate: int
+    input_sample_rate: int
+    output_sample_rate: int
     num_channels: int
     chunk_duration_ms: int
 
     @cached_property
     def samples_per_channel(self) -> int:
-        return int(self.sample_rate * (self.chunk_duration_ms / 1000))
+        return int(self.input_sample_rate * (self.chunk_duration_ms / 1000))
 
     @cached_property
     def bytes_per_channel(self) -> int:
@@ -143,15 +156,76 @@ class IoMode(BaseModel):
 
     @cached_property
     def for_audio_frame(self) -> tuple[int, int, int]:
-        return self.sample_rate, self.num_channels, self.samples_per_channel
+        return self.input_sample_rate, self.num_channels, self.samples_per_channel
+
+    @property
+    def mode_type(self) -> str:
+        """Get mode type string for internal use."""
+        if isinstance(self, WebrtcMode):
+            return "webrtc"
+        elif isinstance(self, WsMode):
+            return "ws"
+        else:
+            raise ConfigurationError(
+                f"Unsupported IO mode: {type(self).__name__}, "
+                f"supported modes are: WebrtcMode, WsMode"
+            )
+
+    def get_io_class(self):
+        """Get the corresponding IO class for this mode."""
+        if isinstance(self, WebrtcMode):
+            from palabra_ai.task.io.webrtc import WebrtcIo
+
+            return WebrtcIo
+        elif isinstance(self, WsMode):
+            from palabra_ai.task.io.ws import WsIo
+
+            return WsIo
+        else:
+            raise ConfigurationError(
+                f"Unsupported IO mode: {type(self).__name__}, "
+                f"supported modes are: WebrtcMode, WsMode"
+            )
+
+    @classmethod
+    def from_string(cls, mode_str: str, **kwargs) -> IoMode:
+        """Create mode instance from string."""
+        if mode_str == "webrtc":
+            return WebrtcMode(**kwargs)
+        elif mode_str == "ws":
+            return WsMode(**kwargs)
+        else:
+            raise ConfigurationError(
+                f"Unsupported mode string: {mode_str}, "
+                f"supported modes are: 'webrtc', 'ws'"
+            )
+
+    @classmethod
+    def from_api_source(cls, source: dict) -> IoMode:
+        """Create mode instance from API source dictionary."""
+        source_type = source.get("type")
+        if source_type == "webrtc":
+            return WebrtcMode()
+        elif source_type == "ws":
+            return WsMode(
+                input_sample_rate=source.get("sample_rate", WS_MODE_INPUT_SAMPLE_RATE),
+                num_channels=source.get("channels", WS_MODE_CHANNELS),
+                # chunk_duration_ms will use default WS_MODE_CHUNK_DURATION_MS
+            )
+        else:
+            raise ConfigurationError(
+                f"Unsupported API source type: {source_type}, "
+                f"supported types are: 'webrtc', 'ws'"
+            )
 
     def __str__(self) -> str:
-        return f"[{self.name}: {self.sample_rate}Hz, {self.num_channels}ch, {self.chunk_duration_ms}ms]"
+        return f"[{self.name}: {self.input_sample_rate}Hz, {self.num_channels}ch, {self.chunk_duration_ms}ms]"
 
 
 class WebrtcMode(IoMode):
     name: str = "webrtc"
-    sample_rate: int = WEBRTC_MODE_SAMPLE_RATE
+    input_sample_rate: int = WEBRTC_MODE_INPUT_SAMPLE_RATE
+    output_sample_rate: int = WEBRTC_MODE_OUTPUT_SAMPLE_RATE
     num_channels: int = WEBRTC_MODE_CHANNELS
     chunk_duration_ms: int = WEBRTC_MODE_CHUNK_DURATION_MS
 
@@ -174,7 +248,8 @@ class WebrtcMode(IoMode):
 
 class WsMode(IoMode):
     name: str = "ws"
-    sample_rate: int = WS_MODE_SAMPLE_RATE
+    input_sample_rate: int = WS_MODE_INPUT_SAMPLE_RATE
+    output_sample_rate: int = WS_MODE_OUTPUT_SAMPLE_RATE
     num_channels: int = WS_MODE_CHANNELS
     chunk_duration_ms: int = WS_MODE_CHUNK_DURATION_MS
 
@@ -185,7 +260,7 @@ class WsMode(IoMode):
                 "source": {
                     "type": "ws",
                     "format": "pcm_s16le",
-                    "sample_rate": self.sample_rate,
+                    "sample_rate": self.input_sample_rate,
                     "channels": self.num_channels,
                 },
             },
@@ -194,7 +269,7 @@ class WsMode(IoMode):
                 "target": {
                     "type": "ws",
                     "format": "pcm_s16le",
-                    "sample_rate": self.sample_rate,
+                    "sample_rate": self.output_sample_rate,
                     "channels": self.num_channels,
                 },
             },
@@ -222,6 +297,7 @@ class Preprocessing(BaseModel):
     pre_vad_dsp: bool = True
     record_tracks: list[str] = []
     auto_tempo: bool = False
+    normalize_audio: bool = False  # Enable loudnorm filters (disabled by default)
 
 
 class SplitterAdvanced(BaseModel):
@@ -418,6 +494,8 @@ class Config(BaseModel):
     deep_debug: bool = Field(default=DEEP_DEBUG, exclude=True)
     timeout: int = Field(default=TIMEOUT, exclude=True)  # TODO!
     trace_file: Path | str | None = Field(default=None, exclude=True)
+    drop_empty_frames: bool = Field(default=False, exclude=True)
+    estimated_duration: float | None = Field(default=None, exclude=True)
 
     def __init__(
         self,
@@ -480,19 +558,7 @@ class Config(BaseModel):
         # Reconstruct mode from input_stream/output_stream
         if "input_stream" in data and "mode" not in data:
             source = data.get("input_stream", {}).get("source", {})
-            source_type = source.get("type")
-
-            if source_type == "webrtc":
-                # WebrtcMode with default parameters
-                data["mode"] = WebrtcMode()
-            elif source_type == "ws":
-                # WsMode with parameters from JSON
-                # Note: chunk_duration_ms is not in API format, use default
-                data["mode"] = WsMode(
-                    sample_rate=source.get("sample_rate", WS_MODE_SAMPLE_RATE),
-                    num_channels=source.get("channels", WS_MODE_CHANNELS),
-                    # chunk_duration_ms will use default WS_MODE_CHUNK_DURATION_MS
-                )
+            data["mode"] = IoMode.from_api_source(source)
 
             # Remove input/output streams as they are generated from mode
             data.pop("input_stream", None)
